#------------------------------------------------------------------------------

# MIT License

# Copyright (c) 2022 Landcare Research New Zealand Ltd

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

#------------------------------------------------------------------------------

# Python 3.7.4 (default, Aug  9 2019, 18:34:13) [MSC v.1915 64 bit (AMD64)]
import numpy as np # version 1.21.0
from nlmpy import nlmpy # version 1.0.1
import matplotlib as mpl # version 3.2.1

#------------------------------------------------------------------------------
# DEFINE BASIC PARAMETERS
#------------------------------------------------------------------------------

# Define domain for data
dim = 640
x = np.linspace(0, 1, dim)

# Set random seed to get the same Perlin noise
np.random.seed(2)

def plotNLM(array, row, col, title):
    axsi = axes[row][col]
    axsi.imshow(perlinNLM, vmin=0, vmax=1, cmap=mpl.cm.plasma)
    axsi.xaxis.set_ticks([])
    axsi.yaxis.set_ticks([])
    axsi.xaxis.set_ticklabels([])
    axsi.yaxis.set_ticklabels([])
    axsi.set_title(title, fontsize=7)

#------------------------------------------------------------------------------
# GENERATE EXAMPLES
#------------------------------------------------------------------------------

mpl.pyplot.rcParams.update({'font.size': 7})
fig, axes = mpl.pyplot.subplots(nrows=3, ncols=3, figsize=(4.5, 6.2)) # figsize=(width, height) in inches

# Example (a)
perlinNLM = nlmpy.perlinNoise(nRow = dim, nCol = dim, 
                              periods = (2, 2),
                              persistence = 0.5, 
                              octaves = 3,
                              lacunarity = 2)
title = "(a)\nperiods = (2, 2)\noctaves = 3\nlacunarity = 2\npersistence = 0.5"
plotNLM(perlinNLM, 0, 0, title)

# Example (b)
perlinNLM = nlmpy.perlinNoise(nRow = dim, nCol = dim, 
                              periods = (2, 2),
                              persistence = 0.5, 
                              octaves = 8,
                              lacunarity = 2)
title = "(b)\nperiods = (2, 2)\noctaves = 8\nlacunarity = 2\npersistence = 0.5"
plotNLM(perlinNLM, 0, 1, title)

# Example (c)
perlinNLM = nlmpy.perlinNoise(nRow = dim, nCol = dim, 
                              periods = (4, 4),
                              persistence = 0.75, 
                              octaves = 5,
                              lacunarity = 3)
title = "(c)\nperiods = (4, 4)\noctaves = 5\nlacunarity = 3\npersistence = 0.75"
plotNLM(perlinNLM, 0, 2, title)

# Example (d)
perlinNLM = nlmpy.perlinNoise(nRow = dim, nCol = dim, 
                              periods = (1, 1),
                              persistence = 0.5, 
                              octaves = 1,
                              lacunarity = 2)
title = "(d)\nperiods = (1, 1)\noctaves = 1\nlacunarity = 2\npersistence = 0.5"
plotNLM(perlinNLM, 1, 0, title)

# Example (e)
perlinNLM = nlmpy.perlinNoise(nRow = dim, nCol = dim, 
                              periods = (6, 6),
                              persistence = 0.2, 
                              octaves = 3,
                              lacunarity = 4)
title = "(e)\nperiods = (6, 6)\noctaves = 3\nlacunarity = 4\npersistence = 0.2"
plotNLM(perlinNLM, 1, 1, title)

# Example (f)
perlinNLM = nlmpy.perlinNoise(nRow = dim, nCol = dim, 
                              periods = (10, 10),
                              persistence = 0.5, 
                              octaves = 1,
                              lacunarity = 2)
title = "(f)\nperiods = (10, 10)\noctaves = 1\nlacunarity = 2\npersistence = 0.5"
plotNLM(perlinNLM, 1, 2, title)

# Example (g)
perlinNLM = nlmpy.perlinNoise(nRow = dim, nCol = dim, 
                              periods = (5, 1),
                              persistence = 0.2, 
                              octaves = 2,
                              lacunarity = 2)
title = "(g)\nperiods = (5, 1)\noctaves = 2\nlacunarity = 2\npersistence = 0.2"
plotNLM(perlinNLM, 2, 0, title)

# Example (h)
perlinNLM = nlmpy.perlinNoise(nRow = dim, nCol = dim, 
                              periods = (2, 6),
                              persistence = 0.6, 
                              octaves = 3,
                              lacunarity = 2)
title = "(h)\nperiods = (2, 6)\noctaves = 3\nlacunarity = 2\npersistence = 0.6"
plotNLM(perlinNLM, 2, 1, title)

# Plot legend
axsi = axes[2][2]
legend = np.zeros((10,100))
for i in range(10):
    legend[i,:] = np.linspace(0, 1, 100)


im = axsi.imshow(legend, vmin=0, vmax=1, cmap=mpl.cm.plasma, extent=[0,1,0.47,0.53])
axsi.yaxis.set_ticks([])
axsi.yaxis.set_ticklabels([])
axsi.set_xlabel("Landscape value")

#------------------------------------------------------------------------------

# Export figure
mpl.pyplot.tight_layout(pad=0.5)
mpl.pyplot.savefig('figure-2.png', dpi=300)

#------------------------------------------------------------------------------








